
SELECT = "SELECT {columns} "

FROM = "FROM {tables} "

AS = "AS {alias} "

WHERE = "WHERE {conditions} "

JOIN = "JOIN ({queryings}) "

ON = "ON {commonalities} "

ORDER = "ORDER BY {orderings} "

GROUP = "GROUP BY {groupings} "
